@@ -11,7 +11,7 @@
 #    under the License.
 
 """
-Command-line interface to the Heat API.
+Command-line interface to the Tuskar API.
 """
 
 from __future__ import print_function
@@ -25,29 +25,48 @@ from tuskarclient import client
 import tuskarclient.common.utils as utils
 from tuskarclient.openstack.common.apiclient import exceptions as exc
 
+logging.basicConfig()
 logger = logging.getLogger(__name__)
 
 
 class TuskarShell(object):
 
-    def __init__(self, raw_args):
+    def __init__(self, raw_args,
+                 argument_parser_class=argparse.ArgumentParser):
         self.raw_args = raw_args
+        self.argument_parser_class = argument_parser_class
+
+        self.partial_args = None
+        self.parser = None
+        self.subparsers = None
+        self._prepare_parsers()
+
+    def _prepare_parsers(self):
+        nonversioned_parser = self._nonversioned_parser()
+        self.partial_args =\
+            nonversioned_parser.parse_known_args(self.raw_args)[0]
+        self.parser, self.subparsers =\
+            self._parser(self.partial_args.tuskar_api_version)
 
     def run(self):
         '''Run the CLI. Parse arguments and do the respective action.'''
 
-        nonversioned_parser = self._nonversioned_parser()
-        partial_args = nonversioned_parser.parse_known_args(self.raw_args)[0]
-        parser = self._parser(partial_args.tuskar_api_version)
+        # run self.do_help() if we have no raw_args at all or just -h/--help
+        if not self.raw_args\
+                or self.raw_args in (['-h'], ['--help']):
+            self.do_help(self.partial_args)
+            return 0
 
-        if partial_args.help or not self.raw_args:
-            parser.print_help()
+        args = self.parser.parse_args(self.raw_args)
+
+        # run self.do_help() if we have help subcommand or -h/--help option
+        if args.func == self.do_help or args.help:
+            self.do_help(args)
             return 0
 
-        args = parser.parse_args(self.raw_args)
         self._ensure_auth_info(args)
 
-        tuskar_client = client.get_client(partial_args.tuskar_api_version,
+        tuskar_client = client.get_client(self.partial_args.tuskar_api_version,
                                           **args.__dict__)
         args.func(tuskar_client, args)
 
@@ -86,12 +105,15 @@ class TuskarShell(object):
 
         :param version: version of Tuskar API (and corresponding CLI
             commands) to use
+        :return: main parser and subparsers
+        :rtype: (Parser, Subparsers)
         '''
         parser = self._nonversioned_parser()
         subparsers = parser.add_subparsers(metavar='<subcommand>')
         versioned_shell = utils.import_versioned_module(version, 'shell')
         versioned_shell.enhance_parser(parser, subparsers)
-        return parser
+        utils.define_commands_from_module(subparsers, self)
+        return parser, subparsers
 
     def _nonversioned_parser(self):
         '''Create a basic parser that doesn't contain version-specific
@@ -99,10 +121,10 @@ class TuskarShell(object):
         version should be used for the versioned full blown parser and
         defining common version-agnostic options.
         '''
-        parser = argparse.ArgumentParser(
+        parser = self.argument_parser_class(
             prog='tuskar',
             description='OpenStack Management CLI',
-            add_help=False
+            add_help=False,
         )
 
         parser.add_argument('-h', '--help',
@@ -180,9 +202,24 @@ class TuskarShell(object):
 
         return parser
 
+    @utils.arg(
+        'command', metavar='<subcommand>', nargs='?',
+        help='Display help for <subcommand>')
+    def do_help(self, args):
+        """Display help about this program or one of its subcommands."""
+        if getattr(args, 'command', None):
+            if args.command in self.subparsers.choices:
+                # print help for subcommand
+                self.subparsers.choices[args.command].print_help()
+            else:
+                raise exc.CommandError("'%s' is not a valid subcommand" %
+                                       args.command)
+        else:
+            # print general help
+            self.parser.print_help()
+
 
 def main():
-    logger.addHandler(logging.StreamHandler(sys.stderr))
     try:
         TuskarShell(sys.argv[1:]).run()
     except exc.CommandError as e:
